@@ -71,6 +71,7 @@
 #include "FileSelP.h"
 #include "MButton.h"
 #include "Vlist.h"
+#include "VlistP.h"
 
 #include "d_memdebug.h"
 
@@ -979,8 +980,9 @@ static void FS_listAction(w, event, params, nparams)
 #  define DECAY_TIME 200
    static int firstposx,posx[HISTORY_POINTS+1],posix;
    static int firstposy,posy[HISTORY_POINTS+1],posiy;
-   static int childx,childy,childw,childh,clipw,cliph;
+   static int childx,childy,childw,childh,clipw,cliph,childyp;
    static int moving=0,scrolling=0;
+   static int startvisible=0;
    static Time to;
    Widget child;
    Widget clip;
@@ -1071,6 +1073,8 @@ static void FS_listAction(w, event, params, nparams)
     firstposx = posx[0] = (int) event->xbutton.x_root;
     firstposy = posy[0] = (int) event->xbutton.y_root;
     to = ((XMotionEvent*)event)->time;
+    startvisible = VlistGetFirstVisible(list);
+    childyp = 0;
   }
   else if (streq(params[0],"scroll")) {
     if (event->type != MotionNotify) goto break_scrolling;
@@ -1134,10 +1138,16 @@ static void FS_listAction(w, event, params, nparams)
 	if (*nparams>=3) absfactor = atof((char*)(params[2]));
 	absfactor = absfactor >= 0 ? (absfactor<=200 ? absfactor : 200) : 0;
 	IIMESSAGE1(absfactor,relfactor)
-        if (REVERSE_SCROLLING) { dx = -dx; dy = -dy; }
+        if (REVERSE_SCROLLING) { dx = -dx; } else { dy = -dy; }
 	childx = (int) (childx-(dx*absfactor)-(relfactor*childw*dx)/clipw);
 	childy = (int) (childy-(dy*absfactor)-(relfactor*childh*dy)/cliph);
-	ClipWidgetSetCoordinates(clip,childx,childy);
+	ClipWidgetSetCoordinates(clip,childx,0);
+	if (childy != childyp)
+	{
+	   childyp=childy;
+           startvisible = VlistGetFirstVisible(list);
+	   VlistMoveFirstVisible(list, startvisible, childy);
+	}
 	childx = (int) child->core.x;
 	childy = (int) child->core.y;
 	childw = (int) child->core.width;
@@ -1166,7 +1176,10 @@ break_scrolling:
     XtTranslateCoords(clip,0,((Position)ph/2),&midx,&midy);
     if (y<midy) sy = (int) (sy + abs(ph-20));
     else  sy = (int) (sy - abs(ph-20));
-    ClipWidgetSetCoordinates(clip,sx,sy);
+    ClipWidgetSetCoordinates(clip,sx,0);
+
+    startvisible = VlistGetFirstVisible(list);
+    VlistMoveFirstVisible(list, startvisible, sy);
   }
   ENDMESSAGE(FS_listAction)
 }
@@ -1284,7 +1297,7 @@ static void changeList(w,list,entries)
 {
   int i,e,l;
   USE_Arg(2);
-  char *d,*s;
+  char *d,*s,*p;
 
   BEGINMESSAGE(changeList)
   i=e=l=0;
@@ -1299,12 +1312,17 @@ static void changeList(w,list,entries)
   d = FS_XtMalloc(e*sizeof(char));
   s[0]='\0';
   i=0;
+  p = s;
   while (i<entries) {
-     strcat(s,list[i]);
-     strcat(s,"\n");
+     size_t len = strlen(list[i]);
+
+     memcpy(p, list[i], len);
+     p += len;
+     *(p++) = '\n';
      d[i]=' ';
-     i++;     
+     i++;
   }
+  *p = '\0';
   d[i]='\0';
 
   RESET_Arg;
@@ -1833,6 +1851,7 @@ static void cb_scroll(w, client_data, call_data)
 {
   Widget p;
   char *s;
+  int startvisible;
 
   BEGINMESSAGE(cb_scroll)
 
@@ -1849,22 +1868,38 @@ SMESSAGE(s)
 SMESSAGE(XtName(p))
 
   {
-    Widget clip=NULL,aaa=NULL,scroll=NULL;
+    Widget clip=NULL,aaa=NULL,scroll=NULL,list=NULL;
     FS_WIDGET p;
     int style = (int)(intptr_t)client_data;
-    if      (s[0] == 'c') { clip = FS_CURCLIP; aaa = FS_CURAAA; scroll = FS_CURSCROLL; }
-    else if (s[0] == 's') { clip = FS_SUBCLIP; aaa = FS_SUBAAA; scroll = FS_SUBSCROLL; }
+    if      (s[0] == 'c') { clip = FS_CURCLIP; aaa = FS_CURAAA; scroll = FS_CURSCROLL; list = FS_CURLIST; }
+    else if (s[0] == 's') { clip = FS_SUBCLIP; aaa = FS_SUBAAA; scroll = FS_SUBSCROLL; list = FS_SUBLIST; }
     else style=0;
     if (style == SCROLL_SCROLLPROC || style == SCROLL_JUMPPROC) {
-      int x,y;
+      int x;
       x = (int) aaa->core.x;
-      if (((int)(intptr_t)client_data)==1) y = (int)(intptr_t) aaa->core.y - (int)(intptr_t)call_data;
-      else                               y = (int)(-*((float*)call_data) * aaa->core.height);
-      ClipWidgetSetCoordinates(clip, x, y);
+      ClipWidgetSetCoordinates(clip, x, 0);
+      if (((int)(intptr_t)client_data)==1) 
+      {
+         int dy = (int)(intptr_t)call_data;
+         int ly = ((VlistWidget)list)->vlist.ydelta;
+
+         /* Just scroll one position less... */
+         if (dy>ly) dy-=ly;
+         if (dy<-ly) dy+=ly;
+	 
+         VlistMoveFirstVisible(list, VlistGetFirstVisible(list), dy);
+         XawScrollbarSetThumb(scroll,VlistScrollPosition(list),
+		      VlistVisibleLength(list,clip->core.height));
+      }
+      else
+      {
+         float *percent = (float *) call_data;
+	 VlistSetFirstVisible(list, (int)(VlistEntries(list)**percent+0.5));
+      }
+
     } else if (style == SCROLL_CLIPREPORT) {
-      float h = (float)aaa->core.height;
-      if (h < 1.0) h = 1.0;
-      XawScrollbarSetThumb(scroll,-(float)aaa->core.y/h,(float)clip->core.height/h);
+      XawScrollbarSetThumb(scroll,VlistScrollPosition(list),
+		      VlistVisibleLength(list,clip->core.height));
     }
   }    
   ENDMESSAGE(cb_scroll)
