@@ -41,6 +41,7 @@
 #include <stdlib.h>
 #include <time.h>
 #include <ctype.h>
+#include <signal.h>
 
 #ifdef HAVE_INTTYPES_H
   #include <inttypes.h>
@@ -91,6 +92,7 @@
 #include "misc_private.h"
 #include "version.h"
 #include "widgets_misc.h"
+#include "VlistP.h"
 
 static char* save_directory = NULL;
 static char* open_directory = NULL;
@@ -156,33 +158,42 @@ cb_newtocScrollbar(w, client_data, call_data)
   Widget w;
   XtPointer client_data, call_data;
 {
-  int x,y;
-
   BEGINMESSAGE(cb_newtocScrollbar)
-  x = (int) newtocControl->core.x;
   if (((int)(intptr_t)client_data)==1) {
     int dy = (int)(intptr_t)call_data;
-    y = (int) newtocControl->core.y - dy;
+    /* Just scroll one position less... */
+    int ly = ((VlistWidget)newtoc)->vlist.ydelta;
+    if (dy>ly) dy-=ly;
+    if (dy<-ly) dy+=ly;
+    VlistMoveFirstVisible(newtoc, VlistGetFirstVisible(newtoc), dy);
   } else {
     float *percent = (float *) call_data;
-    y = (int)(-*percent * newtocControl->core.height);
+    VlistSetFirstVisible(newtoc, (int)(VlistEntries(newtoc)**percent+0.5));
   }
-  ClipWidgetSetCoordinates(newtocClip, x, y);
   ENDMESSAGE(cb_newtocScrollbar)
 }
 
 /*##################################################################*/
-/* cb_newtocClipAdjust */
+/* cb_newtocVisibleAdjust */
 /*##################################################################*/
 
-void cb_newtocClipAdjust(w, client_data, call_data)
+void cb_newtocVisibleAdjust(w, client_data, call_data)
   Widget w;
   XtPointer client_data, call_data;
 {
+  int entries;
+
   BEGINMESSAGE(cb_newtocClipAdjust)
-  XawScrollbarSetThumb(newtocScroll,
-		       -(float)newtocControl->core.y/(float)newtocControl->core.height,
-		       (float)newtocClip->core.height/(float)newtocControl->core.height);
+
+  entries = VlistEntries(newtoc);
+
+  if (entries <= 0)
+    XawScrollbarSetThumb(newtocScroll, 0.0, 1.0);
+  else
+    XawScrollbarSetThumb(newtocScroll,
+			 VlistScrollPosition(newtoc),
+		         VlistVisibleLength(newtoc,newtocClip->core.height)); 
+
   ENDMESSAGE(cb_newtocClipAdjust)
 }
 
@@ -931,6 +942,38 @@ cb_savepos(w, client_data, call_data)
     ENDMESSAGE(cb_savepos)
 }
 
+/*##################################################################*/
+/* cb_presentation */
+/*##################################################################*/
+
+void
+cb_presentation(w, client_data, call_data)
+    Widget w;
+    XtPointer client_data, call_data;
+{
+    int pid;
+    typedef void (*sighandler_t)(int);
+    sighandler_t sigold;
+    
+    BEGINMESSAGE(cb_presentation)
+
+    sigold = signal(SIGCLD, SIG_IGN); /* Avoid zombies */
+    if (!(pid = fork()))
+    {
+       /* We have to close all open file descriptors so the child does not
+          inherit them */
+       int i;
+       for (i=3; i<256; i++) 
+          close(i);
+       execl(gv_bin, "gv", "--presentation", gv_filename, 0);
+       printf("Cannot exec %s\n", gv_bin);
+       exit(1);
+    }
+    signal(SIGCLD, sigold); /* restore signal handler */
+    
+    ENDMESSAGE(cb_presentation)
+}
+
 /*##################################################################*/
 /* cb_redisplay */
 /*##################################################################*/
