@@ -40,8 +40,8 @@
 #define  VERSION_APPLIC_CONTEXT	app_con  			/* the Application Context */
 
 char *versionIdentification[] = {
-        "gv 3.6.6",
-        "September 2008",
+        "gv 3.6.6.90",
+        "January 2009",
 	0
 };
 
@@ -58,6 +58,7 @@ char * copyright = "\
  \n\
  Copyright (C) 1992-1997  Johannes Plass, Timothy O. Theisen  \n\
  Copyright (C) 2004,2005,2006,2007 Jose E. Marchesi \n\
+ Copyright (C) 2007,2008,2009 Markus Steinborn \n\
  \n\
  This program is free software; you can redistribute it and/or modify \n\
  it under the terms of the GNU General Public License as published by \n\
